function [ train, group, test, answer ] = formatinputs( training, testing, classes, maxLength )
%FORMATINPUTS Convert cell arrays to matrices for svm functions.
%   [train, group, test, answer] = FORMATINPUTS(training, testing, classes)
%   FORMATINPUTS(training, testing, classes, maxLength)

train = str2asc([training{1:end}]);
test = str2asc([testing{1:end}]);

widthTrain = size(train, 2);
widthTest = size(test, 2);

group = {};
answer = {};
for i = 1:length(classes)
    % Add instances of the class name for each sample in the class
    groups = arrayfun(@(x) repmat(x, length(training{i}), 1), classes(i), 'un', false);
    group = [group; groups{1:end}];

    answers = arrayfun(@(x) repmat(x, length(testing{i}), 1), classes(i), 'un', false);
    answer = [answer; answers{1:end}];
end

if nargin > 3 && maxLength > 0
    if widthTrain > maxLength
        train = train(:, 1:maxLength);
        widthTrain = size(train, 2);
    end
    if widthTest > maxLength
        test = test(:, 1:maxLength);
        widthTest = size(test, 2);
    end
end

if (widthTrain < widthTest)
    train(:, widthTrain+1:widthTest) = 0;
else
    test(:, widthTest+1:widthTrain) = 0;
end
end

