function [ samples, classes ] = getdata( filename, nExamples, randomize )
%GETDATA Read samples from file into cell arrays.
%   filename - input file
%   nExamples (optional) - vector of number of examples of each class to be
%   selected
%   randomize - whether to select randomly or sequentially
%
%   Return values:
%       samples - cell array of arrays of sample strings
%       classes - cell array of class names

f = fopen(filename, 'rt');

line = fgetl(f);
classes = {};
samples = {};
while (line ~= -1)
    % class separated from sample text by space
    space = strfind(line, ' ');
    class = line(1:space(1)-1);
    sample = line(space(1)+1:end);

    % Find index of class in classes
    classIndex = find(strcmp(class, classes));
    if ~size(classIndex, 2)
        % Add class to classes array
        classes = [classes class];
        classes = sort(classes);
        % Find where new class was inserted
        classIndex = find(strcmp(class, classes));
        % Create sample sub-array for samples of this class
        samples = {samples{1:classIndex-1} {} samples{classIndex:end}};
    end
    
    samples{classIndex} = [samples{classIndex} sample];
    line = fgetl(f);
end

fclose(f);

if nargin >= 2
    % Permute each sample array
    for i = 1:length(classes)
        assert(nExamples(i) <= length(samples{i}), ...
            ['Not enough examples found for class ' int2str(i) '.']);
        if nargin > 2 && randomize
            r = randperm(length(samples{i}));
        else
            r = 1:length(samples{i});
        end
        samples{i} = samples{i}(r(1:nExamples(i)));
    end
end

end

