function K = kernelFn( U, V, fn, k, lambda, varargin)
%kernelFunction   Wrapper for string kernels.
%   Applies the kernel to rows of U and V.

if strcmp(fn, 'SSK-LP')
    fn = @ssk_lp;
    lp = 1;
    m = varargin{1};
elseif strcmp(fn, 'SSK-LPC')
    fn = @ssk_lpc;
    lp = 1;
    m = varargin{1};
else
    fn = @ssk;
    lp = 0;
end

[Um Un] = size(U);
[Vm Vn] = size(V);

training = isequal(U(1,:), V(1,:));

% Truncate vectors
vecs = cell(Um, 1);
for i = 1:Um
    [~, ~, vecs{i}] = find(U(i, :));
end

if ~training
    samples = cell(Vm, 1);
    for i = 1:Vm
        [~, ~, samples{i}] = find(V(i, :));
    end
end

K = zeros(Um, Vm);

% Calculate lengths of feature mappings (K(s,s)) to divide by later
Kvecs = zeros(1, Um);
for i = 1:Um
    if lp
        Kvecs(i) = feval(fn, vecs{i}, vecs{i}, k, lambda, m);
    else
        Kvecs(i) = feval(fn, vecs{i}, vecs{i}, k, lambda);
    end
end

if ~training
    Ksamples = zeros(1, Vm);
    for i = 1:Vm
        if lp
            Ksamples(i) = feval(fn, samples{i}, samples{i}, k, lambda, m);
        else
            Ksamples(i) = feval(fn, samples{i}, samples{i}, k, lambda);
        end
    end
end

display = feature('ShowFigureWindows');
if display
    h = waitbar(0);
end

% Apply K to each vector/sample pair
for v = 1:Um
    if ~training
        if display
            waitbar(v/Um, h, 'Classifying...');
        end
        for s = 1:Vm
            if lp
                value = feval(fn, vecs{v}, samples{s}, k, lambda, m) / ...
                    sqrt(Kvecs(v) * Ksamples(s));
            else
                value = feval(fn, vecs{v}, samples{s}, k, lambda) / ...
                    sqrt(Kvecs(v) * Ksamples(s));
            end
            K(v, s) = value;
        end
    else
        if display
            waitbar((((Um)*(Um-1)-(Um-v)*(Um-v+1)))/(Um*(Um+1)), h, 'Learning...');
        end
        K(v, v) = 1;
        for s = v+1:Um
            tic
            if lp
                value = feval(fn, vecs{v}, vecs{s}, k, lambda, m) / ...
                    sqrt(Kvecs(v) * Kvecs(s));
            else
                value = feval(fn, vecs{v}, vecs{s}, k, lambda) / ...
                    sqrt(Kvecs(v) * Kvecs(s));
            end
            K(v, s) = value;
            K(s, v) = value;
        end
    end
end
if display
    close(h);
end

end

