# Michael Giuffrida - Yale University
# Spring 2012 - CPSC 490
#
# Parse the Reuters dataset into the ModApte split
# VIII.B. The Modified Apte ("ModApte") Split :
# Training Set (9,603 docs): LEWISSPLIT="TRAIN";  TOPICS="YES"
# Test Set (3,299 docs): LEWISSPLIT="TEST"; TOPICS="YES"
# Unused (8,676 docs):   LEWISSPLIT="NOT-USED"; TOPICS="YES"
#                     or TOPICS="NO" 
#                     or TOPICS="BYPASS"

import re
import string
import sys
import random
import fileinput
from xml.dom.minidom import parseString
from xml.parsers.expat import ExpatError

def rootFix(filename):
    rootAdded = False
    for line in fileinput.input(filename, inplace=1):
        if fileinput.lineno() == 2:
            if not line.startswith('<DOCUMENT>'):
                print '<DOCUMENT>'
                rootAdded = True
        print line,

    if rootAdded:
        with open(filename, 'a') as f:
            f.write('</DOCUMENT>\n')

def usage():
    print "Usage: parse.py [-o train_output test_output]"
    exit()

trainFile = "train.txt"
testFile = "test.txt"

for i in range(1, len(sys.argv)):
    if sys.argv[i] == "-o":
        if len(sys.argv) <= i+3:
            usage()
        trainFile, testFile = sys.argv[i+1:i+3]

classes = ('corn','crude') #('acq','earn')

train = []
test = []
for i in range(len(classes)):
    train.append(list())
    test.append(list())

start = 0
count = 21
source = ('lewis/reut2-', '.sgm')
for s in [source[0] + '%(n)03d' % {'n':i} + source[1] for i in range(start,count+1)]:
    print "Reading from " + s
    file = open(s)
    data = file.read()
    file.close()

    # Remove invalid character references
    data = re.sub('&#\d+;','',data)

    # Remove excessive whitespace
    data = re.sub('\s+',' ',data)

    try:
        dom = parseString(data)
    except ExpatError:
        # Fix XML
        insertIndex = data.find('>') + 1
        data = data[0:insertIndex] + '\n<DOCUMENT>' + data[insertIndex:] + '</DOCUMENT>\n'
        dom = parseString(data)

    r = dom.getElementsByTagName('REUTERS')

    # Training Set (9,603 train): LEWISSPLIT="TRAIN";  TOPICS="YES"
    # Test Set (3,299 train): LEWISSPLIT="TEST"; TOPICS="YES"

    for doc in r:
        # Use ModeApte split
        if doc.getAttribute('LEWISSPLIT') not in ('TRAIN', 'TEST') or \
           doc.getAttribute('TOPICS') != 'YES':
            continue
        topics = doc.getElementsByTagName('TOPICS')[0];
        topic = False
        for t in topics.getElementsByTagName('D'):
            name = t.firstChild.data
            if topic is False:
                for i in range(len(classes)):
                    if name == classes[i]:
                        topic = i
                        break
            else:
                if name in classes:
                    topic = False
                    break
        if topic is not False:
            text = doc.getElementsByTagName('TEXT')[0];
            if text.getAttribute('TYPE') != 'BRIEF' and text.getAttribute('TYPE') != 'UNPROC':
                body = doc.getElementsByTagName('BODY');
                body = body[0].firstChild.data
                # Remove punctuation and trim
                body = re.sub('[' + string.punctuation + ']|(Reuter)(?i)', '', body).strip()

                if (doc.getAttribute('LEWISSPLIT') == 'TRAIN'):
                    train[topic].append(body)
                else:
                    test[topic].append(body)
    print len(train[0]) + len(train[1])
    print len(test[0]) + len(test[1])

for i in range(len(classes)):
    random.shuffle(train[i])
    random.shuffle(test[i])

with open(trainFile, 'w') as f:
    f.writelines([classes[i] + " " + doc + "\n" for i in range(len(classes)) for doc in train[i]])
with open(testFile, 'w') as f2:
    f2.writelines([classes[i] + " " + doc + "\n" for i in range(len(classes)) for doc in test[i]])




