function [ stats guess classifier ] = runTest(training, testing, classes, group, answer, nTrain, nTest, kernel)
%RUNTEST Run a test.
%   Train a classifier and test it.

ticID = tic;
classifier = svmtrain(training, group,  'kernel_function', kernel, 'method', 'SMO', 'autoscale', false);
timeTrain = toc(ticID);

fprintf('Training time: %f\n', timeTrain);

% Classify test samples
ticID = tic;
guess = svmclassify(classifier, testing);%svmdecision(testing, classifier);
timeTest = toc(ticID);

fprintf('Testing time: %f\n', timeTest)

% precision = (relevant and retrieved) / retrieved
% recall = (relevant and retrieved) / relevant
% F1 = 2*precision*recall / (precision + recall)

precisions = zeros(1, 2);
recalls = zeros(1, 2);
F1 = zeros(1, 2);

for i = 1:2
    numRetrieved = sum(ismember(guess, classes{i}));
    numRelevant = nTest(i);
    start = sum(nTest(1:i-1)) + 1;
    finish = start + nTest(i) - 1;
    numBoth = sum(ismember(guess(start:finish), classes{i}));
    precision = numBoth / numRetrieved;
    recall = numBoth / numRelevant;
    precisions(i) = precision;
    recalls(i) = recall;
    F1(i) = 2*precision*recall / (precision + recall);
end

stats = {};
stats.Precision = precisions;
stats.Recall = recalls;
stats.F1 = F1;

end
