function value = sskMatlab(str1, str2, k, lambda, time)
%SSKMATLAB   Recursive computation of SSK.
%    Efficient computation of SSK.
%    Slow. Do not use.

narginchk(4, 5);

str1 = uint8(str1);
str2 = uint8(str2);

if ~exist('time', 'var')
    time = false;
end

if (time)
    ticID = tic;
end

% Return the normalized kernel.
value = kernel(str1,str2,k,lambda);

if (time)
    toc(ticID)
end

end

% Unnormalized kernel K_i(s,t)
function value = kernel(s, t, i, lambda)
    len1 = length(s);
    len2 = length(t);

    % Previously calculated values of K' and K''
    kPMap = repmat(-1, [len1, len2, i-1]);
    kPPMap = repmat(-1, [len1, len2, i-1]);

    value = K(len1, len2, i);

    % Compute K_i(s, t)
    function value = K(endS, endT, i)
        if min(endS, endT) < i
            value = 0;
            return
        end
        
        % Compute K_i(sx, t)
        x = s(endS);
        value = K(endS-1, endT, i);

        % Sum for every substring of t ending in tj = x
        for j = 1:endT
            if t(j) == x
                value = value + KPrime(endS-1, j-1, i-1) * power(lambda, 2);
            end
        end
    end

    % Compute K'_i(s,t)
    function value = KPrime(endS, endT, i)
        if i == 0
            value = 1;
            return
        elseif min(endS, endT) < i
            value = 0;
            return
        end
        
        if kPMap(endS, endT, i) ~= -1
            value = kPMap(endS, endT, i);
            return
        end
        
        % Compute K'_i(sx,t) recursively
        x = s(endS);
        

        value = lambda * KPrime(endS-1, endT, i);
        
        value = value + KPPrime(endS, endT, i);

        % Add new kernelMap for these strings
        kPMap(endS, endT, i) = value;
    end

    function value = KPPrime(endS, endT, i)
        % TODO: are these checks necessary?
        if i == 0
            value = 1;
            return
        end
        if min(endS, endT) < i
            value = 0;
            return
        end
        
        x = s(endS);
        

        % Look up KPPrime in map
        if kPPMap(endS, endT, i) ~= -1
            value = kPPMap(endS, endT, i);
            return
        end

        
        % Find the largest u in tu such that x does not occur in u

        if (t(endT) == x)
            value = lambda * (KPPrime(endS, endT-1, i) + lambda * KPrime(endS-1, endT-1, i-1));
        else
            found = false;
            for pos = (endT-1):-1:1
                if t(pos) == x
                    found = true;
                    break;
                end
            end
            
            if ~found
                value = 0;
            else
                value = power(lambda, endT - pos) * KPPrime(endS, pos, i);
            end
        end
                
        % Add new kernelMap for these strings
        kPPMap(endS, endT, i) = value;

        
    end



end




